﻿/* кButton - Button with border color, image and effect pressed
 * 
 * (c) Konstantin V. Kuzminov aka BlackKilkennyCat
 * http://www.kilkennycat.ru
 */

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace ZoomLcdChar
{
    /// <summary>
    /// Это немного доработанная кнопка. Умеет показывать границы какого-либо цвета, в том числе при  наведении курсора
    /// и сдвигает картинку при клике, имитируя нажатие.
    /// </summary>
    public class kButton : Button
    {
        private bool _pressed = false; // Запоминание состояния нажата-отжата.
        private bool _enter = false; // Запоминание состояния "есть ли над кнопкой курсор мыши".
        public kButton()
        { // Задание свойств компонента. Основная цель: устранить всякие мерцания при перерисовке.
            SetStyle(
                ControlStyles.UserPaint |
                ControlStyles.AllPaintingInWmPaint |
                ControlStyles.DoubleBuffer |
                ControlStyles.ResizeRedraw |
                ControlStyles.SupportsTransparentBackColor, true);
        }

        #region properties
        /// <summary>
        /// Цвет границ.
        /// </summary>
        private Color _borderColor = Color.White;
        [DefaultValue(typeof(Color), "White"), Category("Border"), Description("")]
        public Color BorderColor
        {
            get { return _borderColor; }
            set
            {
                _borderColor = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Вкл-выкл отображения границ.
        /// </summary>
        private bool _border = false;
        [DefaultValue(typeof(bool), "false"), Category("Border"), Description("")]
        public bool Border
        {
            get { return _border; }
            set
            {
                _border = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Цвет границ при наведении курсора.
        /// </summary>
        private Color _borderEnterColor = Color.White;
        [DefaultValue(typeof(Color), "White"), Category("Border"), Description("")]
        public Color BorderEnterColor
        {
            get { return _borderEnterColor; }
            set
            {
                _borderEnterColor = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Вкл-выкл отображения границ при наведении курсора.
        /// </summary>
        private bool _borderEnter = false;
        [DefaultValue(typeof(bool), "false"), Category("Border"), Description("")]
        public bool BorderEnter
        {
            get { return _borderEnter; }
            set
            {
                _borderEnter = value;
                Invalidate();
            }
        }
        #endregion

        // Перехват обработки события нажатия кнопки мыши. 
        protected override void OnMouseDown(MouseEventArgs e)
        {
            _pressed = true; // Запоминание, что кнопка мыши нажата.
            Invalidate(); // Перерисовка кнопки.
            base.OnMouseDown(e); // Посылаем это событие дальше, тем, кто тоже хочет его услышать.
        }

        // Перехват обработки события отжатия кнопки мыши. 
        protected override void OnMouseUp(MouseEventArgs e)
        {
            _pressed = false; // Запоминание, что кнопка мыши отжата.
            Invalidate(); // Перерисовка кнопки.
            base.OnMouseUp(e); // Посылаем это событие дальше, тем, кто тоже хочет его услышать.
        }

        // Перехват события ухода курсора с кнопки.
        protected override void OnMouseLeave(EventArgs e)
        {
            _enter = false; // Запоминание, что над кнопкой нет курсора мыши.
            Invalidate(); // Перерисовка кнопки.
            base.OnMouseLeave(e); // Посылаем это событие дальше, тем, кто тоже хочет его услышать.
        }

        // Перехват события появления курсора мыши над кнопкой
        protected override void OnMouseEnter(EventArgs e)
        {
            _enter = true; // Запоминание, что курсор мыши теперь над кнопкой.
            Invalidate(); // Перерисовка кнопки.
            base.OnMouseLeave(e); // Посылаем это событие дальше, тем, кто тоже хочет его услышать.
        }
       
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e); // Разрешаем отрисовать всё базовое-системноею
            e.Graphics.Clear(BackColor); // Очистка области рисования, закрашиванием ее цветом фона.
            /*  Отрисовка изображения. Выражение вида " _pressed ? 1 : 0" равносильно "если _pressed (равно истине), то тогда 1, иначе 0"
             *  Такие выражения называются "тернарная услоовная операция". Сокращают код, но иногда ухудшают читабельность.
             *  Кроме того, обязательно надо иметь ввиду, что некоторые компиляторы не очень хорошо понимают их, и при компиляции код
             *  может отличаться от аналогичного "if then else", причём, в худшую сторону. 
             *   _pressed ? 1 : 0 - смещает картинку на 1 пиксель, если кнопка нажата.
             *  (Width - Image.Width) >> 1 и (Height - Image.Height) >> 1) - это позиционирование картинки по середине кнопки.
             */
            if (Image != null) e.Graphics.DrawImage(Image,( _pressed ? 1 : 0) + ((Width - Image.Width) >> 1), (_pressed ? 1 : 0) + ((Height - Image.Height) >> 1));
            if (_border)
            { // отрисовка границ, так же, с эффектом смещения при нажатии
                Pen p = new Pen(_borderColor);
                e.Graphics.DrawRectangle(p, _pressed ? 1 : 0, _pressed ? 1 : 0, ClientSize.Width - 2, ClientSize.Height - 2);
                p.Dispose();
            }
            if (_borderEnter && _enter)
            {   // отрисовка границ при наведении курсора, так же, с эффектом смещения при нажатии
                Pen p = new Pen(_borderEnterColor);
                e.Graphics.DrawRectangle(p, _pressed ? 1 : 0, _pressed ? 1 : 0, ClientSize.Width - 2, ClientSize.Height - 2);
                p.Dispose();
            }
            if (Text.Length > 0)
            { // отрисовка текста
                TextFormatFlags flags = TextFormatFlags.HorizontalCenter | TextFormatFlags.VerticalCenter | TextFormatFlags.WordBreak;
                Rectangle r = ClientRectangle;
                if (_pressed) //эффект смещения при нажатии
                    r.Offset(1, 1);
                TextRenderer.DrawText(e.Graphics, Text, Font, r, ForeColor, flags);
            }
        }
    }
}

